/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* Move computation of board positions */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <stdio.h>
#include <glib.h>
#include "graph.h"
#include "hash.h"
#include "ext.h"
#include "board.h"
#include "compress_board.h"
#include "ask.h"

#if 0
static void print_board(char board[10][11])
{
  int i,j;
  for (j=1;j<8;j++)printf("%5d",board[j][0]);
  printf("\n");
  for (j=9;j>=1;j--)printf("%5d", j);
  printf("\n  +----+----+----+----+----+----+----+----+----+\n");
  for (i=1;i<10;i++)
    {
      printf("%c |",'a'+i-1);
      for (j=9;j>0;j--)
	printf("%3d |",board[j][i]);
      if(i>0 && i<10)printf(" %c",'a'+i-1);
      if(i<10) 
	printf("\n  +----+----+----+----+----+----+----+----+----+");
      printf("\n");
    }
  for (j=9;j>=1;j--)printf("%5d", j);
  printf("\n");
  for (j=1;j<8;j++)printf("%5d",board[j][10]);
  printf("\n");
}

static void print_boards(t_node *n, int lev)
{
  GList *l;
  printf("%d:\n", lev);
  print_board (n->board);
  if (n->mark==0)
    {
      n->mark=1;
      for (l=n->tos; l!=NULL; l=l->next)
	print_boards(((t_arc *)l->data)->to_node, lev+1);
    }
}
#endif

void init_board(char board[10][11])
{
  int i,j;
  for (i=0; i<=9; i++)
    for (j=0; j<=10; j++)
      board[i][j]=0;
  for (i=1;i<10;i+=8)
    for (j=-1; j<2; j+=2)
      {
	board[5+j*4][i]=LANCE;
	board[5+j*3][i]=KNIGHT;
	board[5+j*2][i]=SILVER;
	board[5+j*1][i]=GOLD;
	board[5][i]=KING;
      }
  board[8][2]=ROOK+WHITE;
  board[2][8]=ROOK+BLACK;
  board[2][2]=BISHOP+WHITE;
  board[8][8]=BISHOP+BLACK;
  for (j=1;j<=9;j++)
    {
      for (i=3;i<9;i+=4)
	board[j][i]=PAWN;
      board[j][1]+=WHITE;
      board[j][3]+=WHITE;
      board[j][7]+=BLACK;
      board[j][9]+=BLACK;
    }
  TURN(board)=BLACK;
}

void copy_board (char board2[10][11], char board1[10][11])
{
  int i, j;
  for (i=0;i<=9;i++)
    for (j=0;j<=10;j++)
      board2[i][j]=board1[i][j];
}

#if 0
char nodes_is_equal(t_node *n1, t_node *n2)
{
  int i,j;
  for (i=1;i<=9;i++)
    for (j=0;j<=10;j++)
      if (n1->board[i][j]!=n2->board[i][j]) return (0);
  return (1);
}
#endif

char boards_is_equal(char board1[10][11], char board2[10][11])
{
  int i,j;
  for (i=1;i<=9;i++)
    for (j=0;j<=10;j++)
      if (board1[i][j]!=board2[i][j]) return (0);
  return (1);
}

static char is_promoted (char c)
{
  if ((c&(PIECEMASK|PROMOTED)) == KING) return FALSE;
  return (c&PROMOTED)==PROMOTED;
}

char which_piece (char c)
{
  if ((c&(PIECEMASK|PROMOTED)) == KING) return KING;
  return c&PIECEMASK;
}


static char is_piece(char c)
{
  if (c>='b' && c<='s') c-= ('a'-'A');
  switch (c)
    {
    case 'L':
    case 'N':
    case 'S':
    case 'G':
    case 'R':
    case 'B':
    case 'P':
    case 'K':
	return 1;
    }
  return 0;
}

static int piece_number(char c)
{
  if (c>='b' && c<='s') c-= ('a'-'A');
  switch (c)
    {
    case 'L':
	return LANCE;
    case 'N':
	return KNIGHT;
    case 'S':
	return SILVER;
    case 'G':
	return GOLD;
    case 'R':
	return ROOK;
    case 'B':
	return BISHOP;
    case 'P':
	return PAWN;
    case 'K':
	return KING;
    }
  return 0;
}

static char piece_char(int c)
{
  switch (c)
    {
    case LANCE:
	return 'L';
    case KNIGHT:
	return 'N';
    case SILVER:
	return 'S';
    case GOLD:
	return 'G';
    case ROOK:
	return 'R';
    case BISHOP:
	return 'B';
    case PAWN:
	return 'P';
    case KING:
	return 'K';
    }
  return 0;
}

static char is_xcord(char c) { return (c>='1' && c<='9');}

static char is_ycord(char c) { return (c>='a' && c<='i');}

static char is_promote(char c) { return (c=='+');}

static char dont_promote(char c) { return (c=='-');}

static char is_drop(char c) { return (c=='\'' || c=='*');}

static char is_to(char c) { return (c=='-');}

static char is_capture(char c) { return (c=='x');}

static char one_move(int x1, int y1, int x2, int y2)
{
  if (x1>x2+1 || x1+1<x2) return (0);
  if (y1>y2+1 || y1+1<y2) return (0);
  return (1);
}

char illegal_move(int x1, int y1, int x2, int y2, 
		  int piece, char board[10][11])
{
  
  int i, z1, z2;

  /* Drop */

  if (y1==0 || y1==10)
    {
      if((TURN(board)==BLACK) == (y1==0)) return(1);
      if (x2<1 || y2<1 || x2>9 || y2>9) return (1);
      switch (x1)
	{
	case KNIGHT:
	  if (TURN(board)==WHITE && y2==8) return (52);
	  if (TURN(board)==BLACK && y2==2) return (53);
	  /* No Break */
	case LANCE: 
	case PAWN:
	  if (TURN(board)==WHITE && y2==9) return (54);
	  if (TURN(board)==BLACK && y2==1) return (55);
	}
      if(board[x2][y2]!=0) return (58);
      return (0);
    }

  /* Inside the border */
  if (x1<1 || x2<1 || y1<1 || y2<1 || x1>9 || x2>9 || y1>9 || y2>9)
    return (1);

  /* The correct piece */
  if (which_piece(board[x1][y1])!=piece) return (2);

  /* Source and destination are equal */
  if (x1==x2 && y1==y2) return (3);

  /* The source piece must be of correct color */
  if ((board[x1][y1]&BLACK)!=TURN(board)) return (4);

  /* The destination must either be blank or contain 
     a piece of opposite color */
  if (board[x2][y2]!=0)
    {
      if ((board[x2][y2]&BLACK)==TURN(board)) return (5);
    }

  /* Flip view if the piece is BLACK */
  if (TURN(board)==WHITE)
    {
      z1=y1; z2=y2;
    } 
  else
    {
      z1=10-y1; z2=10-y2;
    }

  switch (board[x1][y1]&(PIECEMASK|PROMOTED))
    {
    case LANCE:
      if (x1 != x2) return (6);
      if (z1>z2) return (7);
      if (y1<y2)
	for (i=y1+1; i<y2; i++) 
	  { if (board[x1][i]!=0) return (8);}
      else
	for (i=y2+1; i<y1; i++) 
	  if (board[x1][i]!=0) return (9);
      break;
    case KNIGHT:
      if (z1+2!=z2) return (10);
      if (x1+1!=x2 && x1!=x2+1) return (11);
      break;
    case SILVER:
      if (!one_move(x1, y1, x2, y2)) return (12);
      if (x1==x2 && z1-1==z2) return (28);
      if (y1==y2) return (29);
      break;
    case PAWN|PROMOTED:
    case SILVER|PROMOTED:
    case KNIGHT|PROMOTED:
    case LANCE|PROMOTED:
    case GOLD:
      if (!one_move(x1, y1, x2, y2)) return (13);
      if (x1!=x2 && z1-1==z2) return (30);
      break;
    case ROOK:
      if (x1!=x2 && y1!=y2) return (14);
    case ROOK|PROMOTED:
      if (!one_move(x1, y1, x2, y2))
	{
	  if(x1==x2)
	    {
	      if (y1<y2)
		for (i=y1+1; i<y2; i++) 
		  { if (board[x1][i]!=0) return (15);}
	      else
		for (i=y2+1; i<y1; i++) 
		  if (board[x1][i]!=0) return (16);
	    } 
	  else if (y1==y2)
	    {
	      if (x1<x2)
		for (i=x1+1; i<x2; i++) 
		  { if (board[i][y1]!=0) return (17);}
	      else
		for (i=x2+1; i<x1; i++) 
		  if (board[i][y1]!=0) return (18);
	    }
	  else return (19);
	}
      break;
    case BISHOP:
      if (x1==x2 || y1==y2) return (20);
    case BISHOP|PROMOTED:
      if (!one_move(x1, y1, x2, y2))
	{
	  if (x1-x2==y1-y2)
	    {
	      if (x1<x2)
		for (i=1; i<x2-x1; i++) 
		  { if (board[x1+i][y1+i]!=0) return (21);}
	      else
		for (i=1; i<x1-x2; i++) 
		  if (board[x2+i][y2+i]!=0) return (22);
	    }
	  else if (x2-x1==y1-y2)
	    {
	      if (x1<x2)
		for (i=1; i<x2-x1; i++) 
		  { if (board[x1+i][y1-i]!=0) return (23);}
	      else
		for (i=1; i<x1-x2; i++) 
		  if (board[x2+i][y2-i]!=0) return (24);
	    }
	  else return (25);
	}
      break;
    case PAWN:
      if (x1 != x2) return (26);
      if (z1+1!=z2) return (27);
      break;
    case KING:
      if (!one_move(x1, y1, x2, y2)) return (28);
      break;
    }
  return (0);
}

char *text_move(int x1, int y1, int x2, int y2, 
		int piece, char board[10][11], char auto_promote)
{
  char capture='-';
  char promote='\0';
  char move[10];

  if(y1==0 || y1==10)
    sprintf(move, "%c'%d%c", piece_char(x1), x2, 'a'+y2-1);
  else 
    {
      if (piece!=KING && piece!=GOLD && 
	  !(board[x1][y1]&PROMOTED))
	{
	  if (TURN(board)==BLACK && (y1<4 || y2<4)) promote='+';
	  if (TURN(board)==WHITE && (y1>6 || y2>6)) promote='+';
	}

      if (promote=='+' && !auto_promote && 
	  ((piece==LANCE && y2>=2 && y2<=8) ||
	   (piece==PAWN && y2>=2 && y2<=8) ||
	   (piece==KNIGHT && y2>=3 && y2<=7) ||
	   piece==SILVER || piece==ROOK || piece==BISHOP))
	{
	  if (!ask_question(_("Promote?"))) promote='\0';
	}

      if (board[x2][y2]) capture='x';
      sprintf(move, "%c%d%c%c%d%c%c", piece_char(piece),
	      x1, 'a'+y1-1, capture, x2, 'a'+y2-1, promote);
    }
  return(find_string_in_hash_table(move));
}


char move_board(char board1[10][11], char board2[10][11], char *move, 
		t_cord *from, t_cord *to, t_cord *extra)
{
  int x1= -1, x2, y1= -1, y2, i, j;
  int piece;
  int drop=0;
  int capture=0;
  int promote=0;
  int promoted_piece=0;
  int found_move=0;
  char *p;

  extra->x= extra->y= 0;

  /* Analyze move */
  p=move;
  if (is_promote(*p)) { p++;  promoted_piece=1;}
  if (is_piece(*p)) { piece=piece_number(*p); p++;}
  else return (67);
  if (is_drop(*p)) { drop=1; p++;}
  else if (is_capture(*p)) { capture=1; p++;}
  if (is_ycord(*p)) { y1=*p-'a'+1; p++;}
  if (is_xcord(*p)) { x2=*p-'0'; p++;}
  else return (66);
  if (is_xcord(*p)) { x1=x2; x2=*p-'0'; p++;}
  if (is_ycord(*p)) { y2=*p-'a'+1; p++;}
  else return (65);
  if (is_to(*p) || is_capture(*p) || is_xcord(*p)) 
    { 
      if (is_capture(*p)) { capture=1; p++;} else
      if (is_to(*p)) p++;
      x1=x2; x2=*p-'0'; p++;
      if (is_ycord(*p)) { y1=y2; y2=*p-'a'+1; p++;} else return (50);
    }
  else if (is_ycord(*p)) { y1=y2; y2=*p-'a'+1; p++;}
  if (is_promote(*p)) { promote=1; p++;}
  if (dont_promote(*p)) { promote= -1; p++;}
  if (drop && promote) return (51);
  if (drop || promote== -1)
    {
      switch (piece)
	{
	case KNIGHT:
	  if (TURN(board1)==WHITE && y2==8) return (52);
	  if (TURN(board1)==BLACK && y2==2) return (53);
	  /* No Break */
	case LANCE: 
	case PAWN:
	  if (TURN(board1)==WHITE && y2==9) return (54);
	  if (TURN(board1)==BLACK && y2==1) return (55);
	}
    }
  if (drop && (x1!= -1 || y1!= -1))return (56);

  /* Copy Board */

  copy_board (board2, board1);
  if (TURN(board1)==BLACK) TURN(board2)=WHITE;
  else TURN(board2)=BLACK;

  if (drop)
    {
    try_drop:
      if (board2[from->x=piece]
	  [from->y=(TURN(board1)==WHITE?WHITE_HAND:BLACK_HAND)]-- ==0)
	return (57);
      if(board2[x2][y2]!=0) return (58);
      board2[to->x=x2][to->y=y2]=
	piece|(TURN(board1)==WHITE?WHITE:BLACK);
    }
  else
    {
      /* Find x1, y1 */
      if (x1== -1 && y1== -1)
	{
	  for (i=1; i<=9; i++)
	    for (j=1; j<=9; j++)
	      if (!illegal_move(i, j, x2, y2, piece, board1))
		{
		  
		  if (is_promoted(board1[i][j])
		      == promoted_piece)
		    {
		      x1=i; y1=j;
		      found_move=1;
		    }
		}
	}

      /* Find y1 */
      else if (y1== -1)
	{
	  for (j=1; j<=9; j++)
	      if (!illegal_move(x1, j, x2, y2, piece, board1))
		{
		  if (found_move) return (60);
		  found_move=1;
		  y1=j;
		}
	}

      /* Find X1 */
      else if (x1== -1)
	{
	  for (i=1; i<=9; i++)
	    if (!illegal_move(i, y1, x2, y2, piece, board1))
	      {
		if (found_move) return (61);
		found_move=1;
		x1=i;
	      }
	}
      else if (!illegal_move(x1, y1, x2, y2, piece, board1))
	found_move=1;
      else
	return illegal_move(x1, y1, x2, y2, piece, board1);

      /* Is promote legal */
      if (promote && TURN(board1)==WHITE && y1<7 && y2<7) return (62);
      if (promote && TURN(board1)==BLACK && y1>3 && y2>3) return (63);

      /* Is there a legal move */
      if (!found_move) goto try_drop;/*return (64);*/

      /* Do the move */
      if (board2[x2][y2]!=0)
	board2[extra->x= which_piece(board2[x2][y2])]
	  [extra->y=(TURN(board1)==WHITE?WHITE_HAND:BLACK_HAND)]++;
      board2[to->x=x2][to->y=y2]=board2[x1][y1];
      board2[from->x=x1][from->y=y1]=0;
      if (promote==1) board2[x2][y2]|=PROMOTED;
    }
  return (0);
}

static char compute_boards_rek(t_node *n, char board[10][11])
{
  char board2[10][11];
  int err;
  t_arc *a;
  GSList *l;
  n->mark=1;
  for (l=n->tos; l!=NULL;l=l->next)
    {
      a= (t_arc *)l->data;
      if (a->to_node->mark==0)
	{
	  if ((err=move_board(board, board2, a->move, &a->from, 
			      &a->to, &a->extra)))
	    {
	      char s[1000];
	      sprintf(s, _("Illegal move in database at line %d (%d) %s\n"),
		     a->line, err, a->move);
	      display_error(s);
	      return (err);
	    }
#ifdef COMPRESS_BOARD
	  compress_board(board2, a->to_node->board);
#else
	  copy_board(a->to_node->board, board2);
#endif
	  if ((err=compute_boards_rek(a->to_node, board2))) 
	    return (err);
	}
      else
	{
#if COMPRESS_BOARD
	  int cboard2[9];
#endif
	  if ((err=move_board(board, board2, a->move, &a->from, 
			      &a->to, &a->extra)))
	    {
	      char s[1000];
	      sprintf(s, _("Illegal move in database (%d) %s\n"),
		      err, a->move);
	      display_error(s);
	      return (err);
	    }
#ifdef COMPRESS_BOARD
	  compress_board(board2, cboard2);
	  if (!cboards_is_equal (a->to_node->board, cboard2))
#else
	  if (!boards_is_equal (a->to_node->board, board2))
#endif
	    {
	      display_error(_("Database is inconsistent\n"));
	      return (100);
	    }
	}
    }
  return (0);
}

t_graph *compute_boards(t_graph *g)
{
  char board[10][11];
  init_board (board);
#ifdef COMPRESS_BOARD
  compress_board (board, g->main->board);
#else
  copy_board (g->main->board, board);
#endif
  if (compute_boards_rek(g->main, board))
    {
      free_graph (g); 
      return (NULL);
    }
  mark_nodes(g->main, 0);
  return (g);
}

