/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "graph.h"
#include "compress_board.h"

char cboards_is_equal(int board1[9], int board2[9])
{
  int i;
  for (i=0;i<=8;i++)
      if (board1[i]!=board2[i]) return (0);
  return (1);
}

void copy_cboard (int cboard2[9], int cboard1[9])
{
  int i;
  for (i=0;i<=8;i++)
    cboard2[i]=cboard1[i];
}



void compress_board (char board[10][11], unsigned int cboard[9])
{
  int i,j;
  int b_index=0;
  int s_index=0;

  for (i=1; i<=9; i++)
    {
      cboard[i-1]=
	(board[i][1]?1U<<31:0)|
	(board[i][2]?1<<30:0)|
	(board[i][3]?1<<29:0)|
	(board[i][4]?1<<28:0)|
	(board[i][5]?1<<27:0)|
	(board[i][6]?1<<26:0)|
	(board[i][7]?1<<25:0);
    }
  cboard[8]|=
    (board[1][8]?1<<24:0)|
    (board[1][9]?1<<23:0)|
    (board[2][8]?1<<22:0)|
    (board[2][9]?1<<21:0)|
    (board[3][8]?1<<20:0)|
    (board[3][9]?1<<19:0)|
    (board[4][8]?1<<18:0)|
    (board[4][9]?1<<17:0)|
    (board[5][8]?1<<16:0)|
    (board[5][9]?1<<15:0)|
    (board[6][8]?1<<14:0)|
    (board[6][9]?1<<13:0)|
    (board[7][8]?1<<12:0)|
    (board[7][9]?1<<11:0)|
    (board[8][8]?1<<10:0)|
    (board[8][9]?1<<9:0)|
    (board[9][8]?1<<8:0)|
    (board[9][9]?1<<7:0)|
    TURN(board);

  for (i=1; i<=9; i++)
    for (j=1; j<=9; j++)
      {
	if (board[i][j])
	  {
	    cboard[b_index]|= board[i][j]<<s_index;

	    if (s_index == 20)
	      {
		s_index= 0;
		b_index++;
	      }
	    else
	      s_index+= 5;
	  }
      }	  

  for (i=1; i<=7; i++)
    {
      if (board[i][10] || board[i][0])
	{
	  cboard[b_index]|= board[i][0]<<s_index;
	  
	  if (s_index == 20)
	    {
	      s_index= 0;
	      b_index++;
	    }
	  else
	    s_index+= 5;
	}
    }	  
}

void uncompress_board (char board[10][11], unsigned int cboard[9])
{
  int i, j;
  int b_index=0;
  int s_index=0;
  int n_pieces[8];
  unsigned int bboard[10][10], bb=0;

  n_pieces[PAWN]= 18;
  n_pieces[ROOK]= 2;
  n_pieces[BISHOP]= 2;
  n_pieces[LANCE]= 4;
  n_pieces[KNIGHT]= 4;
  n_pieces[SILVER]= 4;
  n_pieces[GOLD]= 4;
  
  for (i=1; i<=9; i++)
    {
      bboard[i][1]= cboard[i-1]&1U<<31;
      bboard[i][2]= cboard[i-1]&1<<30;
      bboard[i][3]= cboard[i-1]&1<<29;
      bboard[i][4]= cboard[i-1]&1<<28;
      bboard[i][5]= cboard[i-1]&1<<27;
      bboard[i][6]= cboard[i-1]&1<<26;
      bboard[i][7]= cboard[i-1]&1<<25;
    }
  bboard[1][8]= cboard[8]&1<<24;
  bboard[1][9]= cboard[8]&1<<23;
  bboard[2][8]= cboard[8]&1<<22;
  bboard[2][9]= cboard[8]&1<<21;
  bboard[3][8]= cboard[8]&1<<20;
  bboard[3][9]= cboard[8]&1<<19;
  bboard[4][8]= cboard[8]&1<<18;
  bboard[4][9]= cboard[8]&1<<17;
  bboard[5][8]= cboard[8]&1<<16;
  bboard[5][9]= cboard[8]&1<<15;
  bboard[6][8]= cboard[8]&1<<14;
  bboard[6][9]= cboard[8]&1<<13;
  bboard[7][8]= cboard[8]&1<<12;
  bboard[7][9]= cboard[8]&1<<11;
  bboard[8][8]= cboard[8]&1<<10;
  bboard[8][9]= cboard[8]&1<<9;
  bboard[9][8]= cboard[8]&1<<8;
  bboard[9][9]= cboard[8]&1<<7;
  TURN(board)= CTURN(cboard);

  for (i=1; i<=9; i++)
    for (j=1; j<=9; j++)
      {
	if (bboard[i][j])
	  {
	    if (s_index == 0) bb= cboard[b_index];
	    else bb >>= 5;
	    n_pieces[(board[i][j]= bb & 31)&7]--;

	    if (s_index == 20)
	      {
		s_index= 0;
		b_index++;
	      }
	    else
	      {
		s_index+= 5;
	      }
	  }
	else
	  board[i][j]= 0;
      }
  for (i=1; i<=7; i++)
    {
      if (n_pieces[i])
	{
	  if (s_index == 0) bb= cboard[b_index];
	  else bb >>= 5;
	  board[i][10]= n_pieces[i] - (board[i][0]= bb & 31);
	  
	  if (s_index == 20)
	    {
	      s_index= 0;
	      b_index++;
	    }
	  else
	    {
	      s_index+= 5;
	      cboard[b_index] >>= 5;
	    }
	}
      else
	board[i][0]=board[i][10]= 0;
    }
}
