/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "graph.h"
#include "draw.h"
#include "board.h"
#include <string.h>
#include "hash.h"
#include "ask.h"
#include "fileio.h"
#include "compress_board.h"

#define NLIST_LENGTH 16381
GSList *nlist[NLIST_LENGTH];
#define HLIST_LENGTH 16381
GSList *hlist[HLIST_LENGTH];

/* Hash by string */

struct      GStringChunk;
 GStringChunk* g_string_chunk_new            (gint size);
 gchar*      g_string_chunk_insert           (GStringChunk *chunk,
                                              const gchar *string);
 gchar*      g_string_chunk_insert_const     (GStringChunk *chunk,
                                              const gchar *string);
 void        g_string_chunk_free             (GStringChunk *chunk);

GStringChunk *gsc;
GAllocator *ga;

void hash_string_init()
{
  ga=g_allocator_new("string", 128);
  g_slist_push_allocator(ga);
  gsc= g_string_chunk_new(4096);
  g_slist_pop_allocator();
}

void hash_string_final()
{
  g_slist_push_allocator(ga);
  g_string_chunk_free(gsc);
  g_slist_pop_allocator();
}

char *find_string_in_hash_table(char *s)
{
  gchar *s2;
  g_slist_push_allocator(ga);
  s2= g_string_chunk_insert_const (gsc, s);
  g_slist_pop_allocator();
  return (s2);
}

/* Hash by node_number */

void hash_number_init()
{
  return;
}

void hash_number_final()
{
  int i;
  for (i=0;i<NLIST_LENGTH; i++)
    {
      if (nlist[i]) g_slist_free(nlist[i]);
      nlist[i]= NULL;
    }
}

#if 0
void delete_node_in_number_table(t_node *n)
{
  int h= n->node_number &(NLIST_LENGTH-1);
  nlist[h]= g_slist_remove (nlist[h], n);
}

void delete_nodes_in_number_table(t_node *n)
{
  GSList *l;
  if (n->node_number!=0)
    {
      n->node_number= 0;
      for (l=n->tos; l!=NULL; l=l->next)
	{
	  delete_nodes_in_number_table (((t_arc *)l->data)->to_node);
	}
      delete_node_in_number_table(n);
    }
}
#endif

t_node *new_node_by_number(t_graph *g, int number, char *note)
{
  t_node *n;
  int h;
  h= number&(NLIST_LENGTH-1);
  n= new_node(g, number, note);
  nlist[h]= g_slist_prepend(nlist[h], n);
  return n;
}

t_node *find_node_by_number(int number)
{
  GSList *l;
  for (l=nlist[number&(NLIST_LENGTH-1)];
       ((t_node *)l->data)->node_number!=number;
       l=l->next);
  return ((t_node *)l->data);
}

/* Hash by board_position */

static void hash_init()
{
  int i;
  for (i=0;i<HLIST_LENGTH; i++)
    {
      hlist[i]= NULL;
    }
}

#if COMPRESS_BOARD
static int hash_board(int cboard[9])
{
  int i, s=0;
  for (i=0; i<=8; i++)
     s ^= cboard[i];
  if (s<0) s= -s;
  s= s % HLIST_LENGTH;
  return (s);
}  
#else
static int hash_board(char board[10][11])
{
  int i,j,s=0;
  for (j=2; j<=8; j+=2)
    for (i=1; i<=9; i++)
      s= (s* 1220703125L + board[i][j])|1;
  if (s<0) s= -s;
  s= s % HLIST_LENGTH;
  return (s);
}
#endif

void delete_node_in_hash_table(t_node *n)
{

  int h= hash_board(n->board);
  hlist[h]= g_slist_remove (hlist[h], n);
}

/* A copy of the node is created */
static t_node *insert_node_in_hash_tablex(t_graph *g, 
#ifdef COMPRESS_BOARD
					  int board[9], 
#else
					  char board[10][11], 
#endif
					  int h, char *note)
{
  t_node *n2;
  n2= new_node (g, 0, note);
#ifdef COMPRESS_BOARD
  copy_cboard (n2->board, board);
#else
  copy_board (n2->board, board);
#endif
  hlist[h]= g_slist_prepend(hlist[h], n2);
  return (n2);
}

/* If an equal node does not exist, a copy of the node is created */
t_node *find_node_in_hash_table(t_graph *g, 
#ifdef COMPRESS_BOARD
				int board[9], 
#else
				char board[10][11], 
#endif
				char *note)
{
  GSList *l;
  int h=hash_board(board);
  for(l=hlist[h]; l!=NULL; l=l->next)
    {
#ifdef COMPRESS_BOARD
      if (cboards_is_equal(((t_node *)l->data)->board, board)) 
#else
      if (boards_is_equal(((t_node *)l->data)->board, board)) 
#endif
	return ((t_node *)l->data);
    }
  return(insert_node_in_hash_tablex(g, board, h, note));
}

static void insert_nodes_in_hash_table_rek(t_graph *g, t_node *n)
{
  GSList *l;
  int h;
  static int counter;
  if (n->mark==0)
    {
      n->mark=1;
      h= hash_board(n->board);
      hlist[h]= g_slist_prepend(hlist[h], n);
      for (l=n->tos; l!=NULL; l=l->next)
	{
	  insert_nodes_in_hash_table_rek(g, ((t_arc *)l->data)->to_node);
	}
    }
  if ((counter++ & 2<<11) == 0) update_interface(g);
}

/* NB: The nodes are inserted without copying */
t_node *insert_nodes_in_hash_table(t_graph *g, t_node *n)
{
  hash_init();
  insert_nodes_in_hash_table_rek(g, n);
  mark_nodes(n,0);
  return (n);
}

t_graph *check_duplicate_nodes (t_graph *g)
{
  GSList *l1, *l2;
  t_node *n1, *n2;
  int i;
  for (i= 0; i<HLIST_LENGTH; i++)
    {
      for (l1= hlist[i]; l1!=NULL; l1= l1->next)
	for (l2= l1->next; l2!=NULL; l2= l2->next)
	  {
	    n1= (t_node *)l1->data;
	    n2= (t_node *)l2->data;
#ifdef COMPRESS_BOARD
	    if (cboards_is_equal (n1->board, n2->board))
#else
	    if (boards_is_equal (n1->board, n2->board))
#endif
	      {
		char s[100];
		sprintf(s, _("Duplicate nodes in database (%d %d)\n"),
			n1->node_number, n2->node_number);
		display_error(s);
		free_graph (g);
		return NULL;
	      }
	  }
      if ((i & 2<<7) == 0) update_interface(NULL);
    }
  return g;
}

