/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "graph.h"
#include "move.h"
#include "draw.h"
#include "compress_board.h"

char auto_flip_on;
char auto_move_on;

void auto_move()
{
  auto_move_on= !auto_move_on;
}

void auto_flip()
{
  auto_flip_on= !auto_flip_on;
}

void move_start(t_graph *g)
{
  t_node *n= g->current_node;
  g->current_node= g->main;
#ifdef COMPRESS_BOARD
  if (auto_flip_on && CTURN(g->current_node->board) != CTURN(n->board))
#else
  if (auto_flip_on && TURN(g->current_node->board) != TURN(n->board))
#endif
    flip_view(g);
  else
    draw_board(g, 0, 0);
}


static char move_backwardx(t_graph *g, t_node *n)
{
  if (n==NULL) if((n= g->current_node->prev) == NULL) return FALSE;
  n->next= g->current_node;
  g->current_node= n;
  return TRUE;
}

void move_backward(t_graph *g, t_node *n)
{
  if(move_backwardx (g, n))
    {
      if (auto_flip_on) 
	flip_view(g);
      else
	draw_board(g, 0, 0);
    }
}

void move_backward_branch(t_graph *g)
{
  t_node *n= g->current_node;
  if (!g->current_node) return;
  move_backwardx(g, g->current_node->prev);
  while (g->current_node->prev && 
	 g_slist_length(g->current_node->tos)== 1)
    {
      move_backwardx(g, g->current_node->prev);
    }
#ifdef COMPRESS_BOARD
  if (auto_flip_on && CTURN(g->current_node->board) != CTURN(n->board))
#else
  if (auto_flip_on && TURN(g->current_node->board) != TURN(n->board))
#endif
    flip_view(g);
  else
    draw_board(g, 0, 0);
}

static char move_forwardx(t_graph *g, t_node *n)
{
  if (n==NULL) if((n= g->current_node->next) == NULL) return FALSE;
  n->prev= g->current_node;
  g->current_node= n;
  return TRUE;
}

void move_forward(t_graph *g, t_node *n)
{
  if(move_forwardx (g, n))
    {
      if (auto_flip_on) 
	flip_view(g);
      else
	draw_board(g, 0, 0);
    }
}

void move_forward_branch(t_graph *g)
{
  t_node *n= g->current_node;
  if (!g->current_node) return;
  move_forwardx(g, g->current_node->next);
  while (g->current_node->next && 
	 g_slist_length(g->current_node->tos)== 1)
    {
      move_forwardx(g, g->current_node->next);
    }
#ifdef COMPRESS_BOARD
  if (auto_flip_on && CTURN(g->current_node->board) != CTURN(n->board))
#else
  if (auto_flip_on && TURN(g->current_node->board) != TURN(n->board))
#endif
    flip_view(g);
  else
    draw_board(g, 0, 0);
}

void move_end(t_graph *g)
{
  t_node *n= g->current_node;
  int i=0;
  if (!g->current_node) return;
  while (g->current_node->next) 
    {
      move_forwardx(g, g->current_node->next);
      if (i++ == 999) break;
    }
#ifdef COMPRESS_BOARD
  if (auto_flip_on && CTURN(g->current_node->board) != CTURN(n->board))
#else
  if (auto_flip_on && TURN(g->current_node->board) != TURN(n->board))
#endif
    flip_view(g);
  else
    draw_board(g, 0, 0);
}

t_arc *from_arc (t_node *n)
{
  GSList *l;
  for (l= n->froms; l!=NULL; l=l->next)
    if (((t_arc *)l->data)->from_node == n->prev)
      return (t_arc *)l->data;
  return NULL;
}
