%{
/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "scanner.h"
#include <glib.h>
#include "graph.h"
#include "board.h"
#include "hash.h"
#include "ext.h"
#include "ask.h"

  extern int yyline;
  t_graph *graph; 
  gboolean fold;

  extern void yyerror(char *text);
  extern int yylex(); 
  t_data *data;
%}
%union {
        long val;
        char *string;
        GSList *glist;
	t_arc *arc;
        t_node *node;
        t_data *data;
       }
%token NODE AKOR BKOR NOTE FROMS TOS POSITION BOOKDEL NIL
%token SENTE GOTE EVENT DATE ROUND SITE RESULT OPENING
%token NAME EMAIL COUNTRY BLACK_GRADE WHITE_GRADE COMMENT SOURCE EVENT 
%token VENUE PROAM LEAF BAD SENTE GOTE DATE EVENT OPENING RESULT 
%token <string> TEXT
%token <string> MOVE
%token <val> NUM
%type <string> MAYBEE_TEXT
%type <glist> ARCS
%type <arc> ARC 
%type <arc> ONE_GAME
%type <arc> GAME
%type <arc> MOVES
%type <node> GAME_NODE
%type <node> GAMES
%type <string> BPART
%type <val> LEAF_PART BAD_PART
%type <data> DATA_PART DATA_PART2

%%
MAIN	:	HEAD GAME_NODE TAIL
{
  fold= FALSE;
  graph->main= $2;
}
	|	GAMES
{
  fold= TRUE;
  graph->main= $1;
}
	;
GAMES	:	GAMES ONE_GAME
{ $1->tos= g_slist_prepend ($1->tos, $2); }
	|	ONE_GAME
{ 
  t_node *n;
  n= new_node (graph, 0, NULL);
  n->tos= g_slist_prepend(NULL, $1);
  $$= n;
}
	;
ONE_GAME:	
{
  data= new_data (graph);
}
	        GAME_DATA GAME { $$= $3;}
	;
GAME_DATA:	GAME_DATA DATA
	|	DATA
	;
DATA	:	'[' SENTE TEXT ']'   {data->sente=$3;}	
	|	'[' GOTE TEXT ']'    {data->gote=$3;}	
	|	'[' EVENT TEXT ']'   {data->event=$3;}	
	|	'[' DATE TEXT ']'    {data->date=$3;}	
	|	'[' ROUND TEXT ']'
	|	'[' SITE TEXT ']'
	|	'[' RESULT TEXT ']'  {data->result=$3;}	
	|	'[' OPENING TEXT ']' {data->opening=$3;}	
	|	'[' NAME TEXT ']'
	|	'[' EMAIL TEXT ']'
	|	'[' COUNTRY TEXT ']'
	|	'[' BLACK_GRADE TEXT ']'
	|	'[' WHITE_GRADE TEXT ']'
	|	'[' COMMENT TEXT ']'
	|	'[' SOURCE TEXT ']'
	|	'[' EVENT TEXT ']'
	|	'[' VENUE TEXT ']'
	|	'[' PROAM TEXT ']'
	|	'['  TEXT ']'
	;
GAME	:	MOVES
	|	TEXT MOVES
{$$= $2;}
	;
MAYBEE_TEXT:	TEXT
	|
{ $$= NULL; }
	;
MOVES	:	MAYBEE_NUM MOVE MAYBEE_TEXT MOVES
{
  t_node *n;
  n= new_node (graph, 0, $3);
  n->tos= g_slist_prepend(NULL,$4);
  $$= new_arc(graph, $2, n);
}
  

	|	MAYBEE_NUM MOVE MAYBEE_TEXT
{ 
  t_node *n;
  n= new_node (graph, 0, $3);
  n->leaf= 1;
  n->data= data;
  $$= new_arc(graph, $2, n);
}
	;
MAYBEE_NUM:	NUM {}
	|	
	;
HEAD	:	'(' '#' NUM 'A' '(' '(' NIL
	|
	;
ARCS	:	ARCS ARC
{ $$=g_slist_prepend($1, $2); }
	|
{ $$=NULL }
	;
BAD_PART: {$$= 0;}
        | BAD {$$= 1;}
        ;
ARC	:	'(' TEXT BAD_PART GAME_NODE ')'
{ $$= new_arc(graph, $2, $4); $$->bad= $3; }
	;
ARCSX	:	ARCSX ARCX
	|
	;
ARCX	:	'(' TEXT GAME_NODE ')'
	;
FROMS_PART:	FROMS '(' ARCSX ')'
	|	
	;
BOARD_POSITION:	'#' '('
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE SQUARE
		')'
		HAND HAND
	;
SQUARE	:	NIL
	|	'(' PICE COLOR ')'
	;
COLOR	:	'.' ':' 'B'
	|	'.' ':' 'W'
	;
HAND	:	NIL
	|	'(' PICES ')'
	;
PICES	:	PICES PICE
	|	PICE
	;
PICE	:	'#' '\\' 'l'
	|	'#' '\\' 'n'
	|	'#' '\\' 's'
	|	'#' '\\' 'g'
	|	'#' '\\' 'k'
	|	'#' '\\' 'r'
	|	'#' '\\' 'b'
	|	'#' '\\' 'p'
	|	'#' '\\' 'L'
	|	'#' '\\' 'N'
	|	'#' '\\' 'S'
	|	'#' '\\' 'G'
	|	'#' '\\' 'R'
	|	'#' '\\' 'B'
	|	'#' '\\' 'P'
	;
APART	:	AKOR NUM
	|	AKOR NIL
	|
	;
BPART	:	BKOR NUM { $$=NULL;}
	|	BKOR TEXT { $$=$2;}
	|	NOTE TEXT { $$=$2;}
	|
{ $$=NULL;}
	;
LEAF_PART	:	LEAF { $$= 1;}
	|       { $$= 0;}
	;
PPART	:	POSITION '(' BOARD_POSITION ')'
	|
	;
LPART	:	BOOKDEL NUM 
	|
	;
DATA_PART: {$$= NULL;}
        | DATA_PART2 
        ;
DATA_PART2: DATA_PART2 DATA_PART3
        | { $<data>$= new_data(graph); } DATA_PART3 {$$= $<data>1;}
        ;
DATA_PART3: SENTE TEXT { $<data>0->sente= $2; }
        | GOTE TEXT { $<data>0->gote= $2; }
        | DATE TEXT { $<data>0->date= $2; }
        | EVENT TEXT { $<data>0->event= $2; }
        | OPENING TEXT { $<data>0->opening= $2; }
        | RESULT TEXT { $<data>0->result= $2; }
        ;
GAME_NODE:	'#' NUM '=' '#' 'S' '(' NODE 
			APART
			BPART
			LEAF_PART
			DATA_PART
{ t_node *n;
  n= new_node_by_number(graph, $2, $9);
  n->leaf= $10;
  n->data= $11;
  $$= n;
} 
			FROMS_PART
			TOS '(' ARCS ')'
			PPART
			LPART
                        ')' 
{ $<node>12->tos= $16;
  $$=$<node>12;} 
	|	'#' NUM '#'
{ $$= find_node_by_number($2);}
	;
TAIL	: TAIL '('
	| TAIL ')'
	| TAIL '#'
	| TAIL NUM {}
	| TAIL '.'
	| TAIL TEXT {}
	| TAIL NIL
	|
	;

%%

void yyerror(char *text)
{
  char s[1000];
  sprintf (s,_("Error reading database at line %d: %s\n"),yyline, text);
  display_error (s);
}

t_graph *build_graph(char *name, gboolean *foldp)
{
  if (!init_scanner(name))
    {
      char s[5000];
      sprintf(s,_("Could not open %s"), name);
      display_error(s);
      return(NULL);
    }
  graph= new_graph();
  g_slist_push_allocator(graph->gslist);
  hash_number_init();
  yyparse();
  cleanup_scanner();
  *foldp=fold;
  hash_number_final();
  g_slist_pop_allocator();
  if (yynerrs) 
    {
      free_graph (graph);
      return NULL;
    }
  else
    return (graph);
}

