/*  shogiopening
 *  Copyright (C) 2000 Sverre Hvammen Johansen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "interface.h"
#include "support.h"
#include "graph.h"
#include "draw.h"
#include "preferences.h"

GtkWidget *preferences;
char color_xpm_d_table[PIECES_COLORS][12];

static void preferences_set_color_xpm_d_table ()
{
  GtkWidget *backward, *forward_from, *forward_to, *bad_move;
  guint8 red, green, blue, alfa;

  backward= lookup_widget(preferences, "colorpicker_backward");
  forward_from= lookup_widget(preferences, "colorpicker_forward_from");
  forward_to= lookup_widget(preferences, "colorpicker_forward_to");
  bad_move= lookup_widget(preferences, "colorpicker_bad_move");

  gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(backward), &red, &green, &blue, &alfa);
  sprintf(color_xpm_d_table[BLUE], ". c #%02X%02X%02X", red, green, blue);

  gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(forward_from), &red, &green, &blue, &alfa);
  sprintf(color_xpm_d_table[YELLOW], ". c #%02X%02X%02X", red, green, blue);

  gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(forward_to), &red, &green, &blue, &alfa);
  sprintf(color_xpm_d_table[GREEN], ". c #%02X%02X%02X", red, green, blue);
 
  gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(bad_move), &red, &green, &blue, &alfa);
  sprintf(color_xpm_d_table[RED], ". c #%02X%02X%02X", red, green, blue);
}

void preferences_init()
{
  GtkWidget *backward, *forward_from, *forward_to, *bad_move;
  int red, green, blue;
  char *s;
  preferences= create_propertybox1();

  gnome_dialog_close_hides (GNOME_DIALOG (preferences), TRUE);

  backward= lookup_widget(preferences, "colorpicker_backward");
  forward_from= lookup_widget(preferences, "colorpicker_forward_from");
  forward_to= lookup_widget(preferences, "colorpicker_forward_to");
  bad_move= lookup_widget(preferences, "colorpicker_bad_move");

  red= gnome_config_get_int("/shogiopening/color_backward/red");
  green= gnome_config_get_int("/shogiopening/color_backward/green");
  blue= gnome_config_get_int("/shogiopening/color_backward/blue");
  if (red==0 && green==0 && blue==0) blue=255;
  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(backward), red, green, blue, 0);

  red= gnome_config_get_int("/shogiopening/color_forward_from/red");
  green= gnome_config_get_int("/shogiopening/color_forward_from/green");
  blue= gnome_config_get_int("/shogiopening/color_forward_from/blue");
  if (red==0 && green==0 && blue==0) { red=255; green=255;}
  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(forward_from), red, green, blue, 0);

  red= gnome_config_get_int("/shogiopening/color_forward_to/red");
  green= gnome_config_get_int("/shogiopening/color_forward_to/green");
  blue= gnome_config_get_int("/shogiopening/color_forward_to/blue");
  if (red==0 && green==0 && blue==0) green=255;
  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(forward_to), red, green, blue, 0);

  red= gnome_config_get_int("/shogiopening/color_bad_move/red");
  green= gnome_config_get_int("/shogiopening/color_bad_move/green");
  blue= gnome_config_get_int("/shogiopening/color_bad_move/blue");
  if (red==0 && green==0 && blue==0) red=255;
  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER(bad_move), red, green, blue, 0);

  preferences_set_color_xpm_d_table ();

  s= gnome_config_get_string("/shogiopening/default_database/filename");
  if (!s)
    {
      s= g_strdup(PKGDATADIR "/professional-stripped.sdb");
    }
  gtk_entry_set_text(GTK_ENTRY(lookup_widget (preferences, 
					      "combo_entry_default_database")), 
		     s);
  g_free (s);
}

void preferences_set_color ()
{
  GtkWidget *backward, *forward_from, *forward_to, *bad_move;
  guint8 red, green, blue, alfa;

  backward= lookup_widget(preferences, "colorpicker_backward");
  forward_from= lookup_widget(preferences, "colorpicker_forward_from");
  forward_to= lookup_widget(preferences, "colorpicker_forward_to");
  bad_move= lookup_widget(preferences, "colorpicker_bad_move");

  gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(backward), &red, &green, &blue, &alfa);
  gnome_config_set_int("/shogiopening/color_backward/red", red);
  gnome_config_set_int("/shogiopening/color_backward/green", green);
  gnome_config_set_int("/shogiopening/color_backward/blue", blue);

  gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(forward_from), &red, &green, &blue, &alfa);
  gnome_config_set_int("/shogiopening/color_forward_from/red", red);
  gnome_config_set_int("/shogiopening/color_forward_from/green", green);
  gnome_config_set_int("/shogiopening/color_forward_from/blue", blue);

  gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(forward_to), &red, &green, &blue, &alfa);
  gnome_config_set_int("/shogiopening/color_forward_to/red", red);
  gnome_config_set_int("/shogiopening/color_forward_to/green", green);
  gnome_config_set_int("/shogiopening/color_forward_to/blue", blue);
 
  gnome_color_picker_get_i8 (GNOME_COLOR_PICKER(bad_move), &red, &green, &blue, &alfa);
  gnome_config_set_int("/shogiopening/color_bad_move/red", red);
  gnome_config_set_int("/shogiopening/color_bad_move/green", green);
  gnome_config_set_int("/shogiopening/color_bad_move/blue", blue);

  preferences_set_color_xpm_d_table ();
}

void preferences_set_default_database()
{
  GtkWidget *default_database;
  default_database= lookup_widget (preferences, "combo_entry_default_database");
  gnome_config_set_string ("/shogiopening/default_database/filename", 
			   gtk_entry_get_text(GTK_ENTRY(default_database)));
}
